from NamedEntityRecognition.NamedEntitySentence cimport NamedEntitySentence


cdef class NERCorpus(Corpus):

    def __init__(self, fileName=None):
        """
        Another constructor of NERCorpus which takes a fileName of the corpus as an input, reads the
        corpus from that file.

        PARAMETERS
        ----------
        fileName : str
            Name of the corpus file.
        """
        super().__init__()
        if fileName is not None:
            inputFile = open(fileName, "r", encoding="utf8")
            lines = inputFile.readlines()
            for line in lines:
                self.addSentence(NamedEntitySentence(line))

    cpdef addSentence(self, Sentence s):
        """
        addSentence adds a new sentence to the sentences list

        PARAMETERS
        ----------
        s : Sentence
            Sentence to be added.
        """
        self.sentences.append(s)



cdef class AutoNER:

    def __init__(self):
        """
        Constructor for creating Person, Organization, and Location gazetteers in automatic Named Entity Recognition.
        """
        self.personGazetteer = Gazetteer("PERSON", "gazetteer-person.txt")
        self.organizationGazetteer = Gazetteer("ORGANIZATION", "gazetteer-organization.txt")
        self.locationGazetteer = Gazetteer("LOCATION", "gazetteer-location.txt")



cdef class Gazetteer:

    def __init__(self, name: str, fileName: str):
        """
        A constructor for a specific gazetteer. The constructor takes name of the gazetteer and file name of the
        gazetteer as input, reads the gazetteer from the input file.

        PARAMETERS
        ----------
        name : str
            Name of the gazetteer. This name will be used in programming to separate different gazetteers.
        fileName : str
            File name of the gazetteer data.
        """
        cdef str line
        cdef list lines
        self.__name = name
        self.__data = set()
        inputFile = open(fileName, "r", encoding="utf8")
        lines = inputFile.readlines()
        for line in lines:
            self.__data.add(line.strip())

    cpdef str getName(self):
        """
        Accessor method for the name of the gazetteer.

        RETURNS
        -------
        str
            Name of the gazetteer.
        """
        return self.__name

    cpdef bint contains(self, str word):
        """
        The most important method in Gazetteer class. Checks if the given word exists in the gazetteer. The check
        is done in lowercase form.

        PARAMETERS
        ----------
        word : str
            Word to be search in Gazetteer.

        RETURNS
        -------
        bool
            True if the word is in the Gazetteer, False otherwise.
        """
        lowerMap = {
            ord(u'I'): u'ı',
            ord(u'İ'): u'i',
        }
        return word.translate(lowerMap).lower() in self.__data



from __future__ import annotations
from enum import Enum, auto

"""
Enumerator class for Named Entity types.
"""


class NamedEntityType(Enum):
    NONE = auto()
    PERSON = auto()
    ORGANIZATION = auto()
    LOCATION = auto()
    TIME = auto()
    MONEY = auto()

    @staticmethod
    def getNamedEntityType(entityType: str) -> NamedEntityType:
        """
        Static function to convert a string entity type to NamedEntityType type.

        PARAMETERS
        ----------
        entityType : str
            Entity type in string form

        RETURNS
        -------
        NamedEntityType
            Entity type in NamedEntityType form
        """
        for _type in NamedEntityType:
            if entityType == _type.name:
                return _type
        return NamedEntityType.NONE

    @staticmethod
    def getNamedEntityString(namedEntityType: NamedEntityType) -> str:
        """
        Static function to convert a NamedEntityType to string form.

        PARAMETERS
        ----------
        namedEntityType : NamedEntityType
            Entity type in NamedEntityType form

        RETURNS
        -------
        str
            Entity type in string form
        """
        if namedEntityType is None:
            return "NONE"
        return namedEntityType.name



from NamedEntityRecognition.NamedEntityType import NamedEntityType
from NamedEntityRecognition.NamedEntityWord cimport NamedEntityWord


cdef class NamedEntitySentence(Sentence):

    def __init__(self, sentence: str = None):
        """
        Another constructor of NamedEntitySentence. It takes input a named entity annotated sentence in string
        form, divides the sentence with respect to space and sets the tagged words with respect to their tags.

        PARAMETERS
        ----------
        sentence : str
            Named Entity annotated sentence in string form
        """
        if sentence is None:
            super().__init__()
        else:
            namedEntityType = NamedEntityType.NONE
            self.words = []
            wordArray = sentence.split()
            for word in wordArray:
                if len(word) != 0:
                    if word != "<b_enamex":
                        if word.startswith("TYPE=\""):
                            typeIndexEnd = word.index('\"', 6)
                            if typeIndexEnd != -1:
                                entityType = word[6: typeIndexEnd]
                                namedEntityType = NamedEntityType.getNamedEntityType(entityType)
                            if word.endswith("e_enamex>"):
                                candidate = word[word.index('>') + 1: word.index('<')]
                                if len(candidate) != 0:
                                    self.words.append(NamedEntityWord(candidate, namedEntityType))
                                namedEntityType = NamedEntityType.NONE
                            else:
                                candidate = word[word.index('>') + 1:]
                                if len(candidate) != 0:
                                    self.words.append(NamedEntityWord(candidate, namedEntityType))
                        else:
                            if word.endswith("e_enamex>"):
                                candidate = word[0: word.index('<')]
                                if len(candidate) != 0:
                                    self.words.append(NamedEntityWord(candidate, namedEntityType))
                                namedEntityType = NamedEntityType.NONE
                            else:
                                if len(word) != 0:
                                    self.words.append(NamedEntityWord(word, namedEntityType))



from distutils.core import setup
from Cython.Build import cythonize

setup(
    ext_modules=cythonize(["NamedEntityRecognition/*.pyx"],
                          compiler_directives={'language_level': "3"}),
    name='NlpToolkit-NamedEntityRecognition-Cy',
    version='1.0.2',
    packages=['NamedEntityRecognition'],
    package_data={'NamedEntityRecognition': ['*.pxd', '*.pyx', '*.c', '*.py']},
    url='https://github.com/olcaytaner/TurkishNamedEntityRecognition-Cy',
    license='',
    author='olcaytaner',
    author_email='olcaytaner@isikun.edu.tr',
    description='NER Corpus Processing Library',
    install_requires=['NlpToolkit-Corpus-Cy']
)



cdef class NamedEntityWord(Word):

    def __init__(self, name: str, namedEntityType: NamedEntityType):
        """
        A constructor of NamedEntityWord which takes name and nameEntityType as input and sets the corresponding
        attributes

        PARAMETERS
        ----------
        name : str
            Name of the word
        namedEntityType : NamedEntityType
            NamedEntityType of the word
        """
        super().__init__(name)
        self.__namedEntityType = namedEntityType

    def getNamedEntityType(self) -> NamedEntityType:
        """
        Accessor method for namedEntityType attribute.

        RETURNS
        -------
        NamedEntityType
            namedEntityType of the word.
        """
        return self.__namedEntityType
